package player;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;
import java.awt.Point;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.PixelGrabber;

import javax.swing.ImageIcon;

import configuration.Image;

/**
 * Gets the clickable Points from a mask. If the mask is null, each location is clickable. In the mask image, a clickable location is defined by a white pixel (RGB = 255, 255, 255).
 * @author Martin Henke
 *
 */
public class ClickHelper {
	private boolean[][] clickPoints;
	private final int WIDTH;
	private final int HEIGHT;
	
	/**
	 * Initializes the instance of ClickHelper and identifies the clickable points.
	 * @param mask
	 */
	public ClickHelper(Image mask){
		if (mask != null){
			WIDTH = mask.getIconWidth();
			HEIGHT = mask.getIconHeight();
			setPoints(imageToBufferedImage(mask.getImage()));
		} else {
			WIDTH = 0;
			HEIGHT = 0;
		}
	}
	
	/**
	 * Converts a java.awt.Image to BufferedImage.
	 * @param image The image to convert.
	 * @return The BufferedImage representing the specified image.
	 */
	private static BufferedImage imageToBufferedImage(java.awt.Image image) {
		// Create a buffered image with a format that's compatible with the screen
		BufferedImage bimage = null;
		if (image instanceof BufferedImage) {
			bimage = (BufferedImage)image;
		} else {

			// This code ensures that all the pixels in the image are loaded
			image = new ImageIcon(image).getImage();

			// Determine if the image has transparent pixels; for this method's implementation, see Determining If an Image Has Transparent Pixels
			boolean hasAlpha = hasAlpha(image);

			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			try {
				// Determine the type of transparency of the new buffered image
				int transparency = Transparency.OPAQUE;
				if (hasAlpha) {
					transparency = Transparency.BITMASK;
				}

				// Create the buffered image
				GraphicsDevice gs = ge.getDefaultScreenDevice();
				GraphicsConfiguration gc = gs.getDefaultConfiguration();
				bimage = gc.createCompatibleImage(image.getWidth(null), image.getHeight(null), transparency);
			} catch (HeadlessException e) {
				// The system does not have a screen
			}

			if (bimage == null) {
				// Create a buffered image using the default color model
				int type = BufferedImage.TYPE_INT_RGB;
				if (hasAlpha) {
					type = BufferedImage.TYPE_INT_ARGB;
				}
				bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
			}

			// Copy image to buffered image
			Graphics g = bimage.createGraphics();

			// Paint the image onto the buffered image
			g.drawImage(image, 0, 0, null);
			g.dispose();
		}
		
	    return bimage;
	}
	
	/**
	 * Check if an image has transparent pixels.
	 * @param image The image to check.
	 * @return True if an image has transparent pixels, otherwise false.
	 */
	private static boolean hasAlpha(java.awt.Image image) {
		boolean retVal = false;
		// If buffered image, the color model is readily available
		if (image instanceof BufferedImage) {
			BufferedImage bimage = (BufferedImage)image;
			retVal = bimage.getColorModel().hasAlpha();
		} else {

			// Use a pixel grabber to retrieve the image's color model. Grabbing a single pixel is usually sufficient.
			PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
			try {
				pg.grabPixels();
			} catch (InterruptedException e) {
			}

			// Get the image's color model
			ColorModel cm = pg.getColorModel();
			retVal = cm.hasAlpha();
		}
	    return retVal;
	}
	
	/**
	 * Identifies white pixels and sets the result in the clickPoints array.
	 * @param mask The image to check.
	 */
	private void setPoints(BufferedImage mask){
		Color pointColor;
		clickPoints = new boolean[WIDTH][HEIGHT];
		for (int x = 0; x < WIDTH; x++){			
			for (int y = 0; y < HEIGHT; y++){
				pointColor = new Color(mask.getRGB(x, y));
				clickPoints[x][y] = (pointColor.getRed() == 255 && pointColor.getGreen() == 255 && pointColor.getBlue() == 255);
			}
		}
	}
	
	/**
	 * Checks if a click was made on a clickable area.
	 * @param point The location of the click.
	 * @return True if the click was made on a clickable area, otherwise false.
	 */
	public boolean isClick(Point point){
		boolean retVal;
		if (clickPoints == null){
			retVal = true;
		} else {
			int x = point.x;
			int y = point.y;
			if (x > -1 && y > -1 && x < WIDTH && y < HEIGHT){
				retVal = clickPoints[point.x][point.y];
			} else {
				retVal = false;
			}
		}
		return retVal;
	}	
}
