package player;

import java.io.*;
import java.util.Collections;
import java.util.LinkedList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import player.PlayListEntry.SortMethod;

/**
 * The class PlayList is serializable and contains a list with PlayListEntry objects.
 * @author Martin Henke
 *
 */
@XmlRootElement(name="PlayList")
public class PlayList implements Serializable {
	private static final long serialVersionUID = -7879245579754200310L;

	public static final String EXTENSION = ".playlist";
	public static final String DEFAULT_NAME = "default" + EXTENSION;
	private LinkedList<PlayListEntry> entries = new LinkedList<PlayListEntry>();
	private String path = null;
	
	/**
	 * Initializes the PlayList.
	 */
	public PlayList(){ }
	
	/**
	 * Gets the path to the play list file.
	 * @return The path to the play list file.
	 */
	public String getPath() { return path; }
	/**
	 * Sets the path to the play list file.
	 * @param path The path to the play list file.
	 */
	@XmlTransient
	public void setPath(String path) { this.path = path; }
	
	/**
	 * Gets the list with play list entries.
	 * @return The list with play list entries. 
	 */
	public LinkedList<PlayListEntry> getEntries() { return entries; }
	/**
	 * Sets the list with play list entries.
	 * @param entries The list with play list entries.
	 */
	@XmlElement(name="file", required=true)
	public void setEntries(LinkedList<PlayListEntry> entries){
		if (entries != null){
			this.entries = entries;
		} else {
			this.entries = new LinkedList<PlayListEntry>();
		}		
	}
	
	/**
	 * Loads a play list.
	 * @param file The file representing the play list.
	 * @return A PlayList object.
	 * @throws JAXBException
	 */
	public static PlayList playListLoad (File file) throws JAXBException{
		PlayList retVal;
		if (file.exists()){
			JAXBContext jc = JAXBContext.newInstance(PlayList.class);
			Unmarshaller unmarshaller = jc.createUnmarshaller();
	    
			retVal = (PlayList) unmarshaller.unmarshal(file);
		} else {
			retVal = new PlayList();
		}
		retVal.setPath(file.getAbsolutePath());
		return retVal;
	}
	
	/**
	 * Saves the play list.
	 * @throws JAXBException
	 * @throws FileNotFoundException
	 */
	public void playListSave() throws JAXBException, FileNotFoundException{
		if (path != null){
			playListSave(new File(path));
		} else {
			throw new java.io.FileNotFoundException();
		}
	}
	
	/**
	 * Saves the play list at the specified location.
	 * @param file The file pointing to the location to save the play list at.
	 * @throws JAXBException
	 */
	private void playListSave(File file) throws JAXBException{
		JAXBContext context = JAXBContext.newInstance(PlayList.class);		
	    Marshaller marshaller = context.createMarshaller();
	    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);	    
	    marshaller.marshal(this, file);	
	}
	
	/**
	 * Gets the name for saving a play list.
	 * @param file The file pointing to the path to save the play list at.
	 * @return A valid file name (with extension).
	 */
	public static File getPlayListFileName(File file){
		if (file != null){
			if (file.getName().toLowerCase().endsWith(EXTENSION)){
				return file;
			} else {
				return new File(file.getAbsolutePath() + EXTENSION);
			}
		} else {
			return file;
		}
	}
	
	/**
	 * Adds the specified files to the play list.
	 * @param files The files to add to the play list.
	 */
	public void addEntries(File[] files){
		for (File file : files){
			addEntry(file);
		}
	}
	
	/**
	 * Adds a file to the play list.
	 * @param mediaFile The file to add.
	 */
	public void addEntry(File mediaFile){
		if (!mediaFile.isDirectory()){
			if (FileTypes.isSupportedFileExtension(mediaFile)){
				entries.add(new PlayListEntry(mediaFile));
			}
		} else {
			addDirectory(mediaFile);
		}
	}
	
	/**
	 * Adds a directory to the play list.
	 * @param directory The directory to add to the play list.
	 */
	private void addDirectory(File directory){
		for (File file : directory.listFiles()){
			addEntry(file);
		}
	}
	
	/**
	 * Removes an entry from the play list.
	 * @param entry The entry to remove from the play list.
	 */
	public void removeEntry(PlayListEntry entry){
		if (entries.contains(entry)){
			entries.remove(entry);
		}
	}
	
	/**
	 * Gets the number of play list entries.
	 * @return The number of play list entries.
	 */
	public int size(){ return entries.size(); }
	
	/**
	 * Gets the PlayListEntry at the specified index.
	 * @param index The index specifying the location in the list.
	 * @return The PlayListEntry at the specified index.
	 */
	public PlayListEntry getEntry(int index){ return entries.get(index); }
	
	/**
	 * Sorts the list with the specified sort method.
	 * @param sortMethod The sort method to use for sorting.
	 */
	public void sort(SortMethod sortMethod){
		PlayListEntry.sortMethod = sortMethod;
		Collections.sort(entries);
	}	
}
