package player;

import java.io.File;
import java.io.Serializable;

import javax.xml.bind.annotation.XmlAttribute;

/**
 * The class PlayListEntry is serializable and comparable (for sorting).
 * @author Martin Henke
 *
 */
public class PlayListEntry implements Serializable, Comparable<PlayListEntry> {
	
	private static final long serialVersionUID = -9145371403264486621L;
	
	static enum SortMethod{
		FullPath_A_Z, FullPath_Z_A, Name_A_Z, Name_Z_A, FileSize_0_9, FileSize_9_0
	};

	private String fullPath;
	private transient String name = null;
	private transient Long fileSize = (long)0;
	static SortMethod sortMethod = SortMethod.FullPath_A_Z;

	/**
	 * The empty constructor is required for XML serialization and should not be used in your code.
	 */
	public PlayListEntry(){ }
	/**
	 * The constructor initializes the object and sets the path.
	 * @param file The path to the media file.
	 */
	public PlayListEntry(File file){
		setFullPath(file.getAbsolutePath());
	}
	
	/**
	 * Gets the full path to the media file. 
	 * @return The full path to the media file.
	 */
	public String getFullPath() { return fullPath; }
	/**
	 * Sets the full path to the media file and loads the meta data.
	 * @param fullPath The full path to the media file.
	 */
	@XmlAttribute(name="fullPath", required=true)
	public void setFullPath(String fullPath){
		this.fullPath = fullPath;
		loadMetaData();
	}
	
	/**
	 * Loads the meta data name and file size.
	 */
	private void loadMetaData(){
		File f = new File(this.fullPath);
		if (f.exists() && f.isFile()){
			this.name = f.getName();
			this.fileSize = f.length();
		}
	}

	/**
	 * Gets the file name. 
	 * @return The file name.
	 */
	public String getName() { return name; }
	
	/**
	 * Gets the file size.
	 * @return The file size.
	 */
	public Long getFileSize() { return fileSize; }
	
	/**
	 * Compares this instance to another PlayListEntry instance.
	 */
	@Override
	public int compareTo(PlayListEntry o) {
		int retVal = 0;
		switch(sortMethod){
		case FullPath_A_Z:
			retVal = fullPath.compareTo(o.fullPath);
			break;
		case FullPath_Z_A:
			retVal = fullPath.compareTo(o.fullPath) * (-1);
			break;
		case Name_A_Z:
			retVal = name.compareTo(o.name);
			if (retVal == 0){
				retVal = fullPath.compareTo(o.fullPath);
			}
			break;
		case Name_Z_A:
			retVal = name.compareTo(o.name);
			if (retVal == 0){
				retVal = fullPath.compareTo(o.fullPath);
			}
			retVal *=(-1);
			break;
		case FileSize_0_9:
			retVal = fileSize.compareTo(o.fileSize);
			if (retVal == 0){
				retVal = fullPath.compareTo(o.fullPath);				
			}
		break;
		case FileSize_9_0:
			retVal = fileSize.compareTo(o.fileSize);
			if (retVal == 0){
				retVal = fullPath.compareTo(o.fullPath);				
			}
			retVal *=(-1);
			break;
		default:
			break;
		}
		return retVal;
	}
	
	/**
	 * Returns the name
	 */
	@Override
	public String toString(){
		return name;
	}
}
