package player;

import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.AbstractListModel;
import javax.xml.bind.JAXBException;

import player.PlayListEntry.SortMethod;

/**
 * The class PlayListListModel inherits from AbstractListModel and is used in the PlayList window.
 * @author Martin Henke
 *
 */
public class PlayListListModel extends AbstractListModel<PlayListEntry> {

	private static final long serialVersionUID = -4871503408993410728L;
	
	private PlayList playList = new PlayList();
	
	/**
	 * Sets the PlayList object to provide the play list entries.
	 * @param playList The play list with the play list entries. 
	 */
	public void setData(PlayList playList){
		this.playList = playList;
	}
	
	/**
	 * Gets the number of play list entries.
	 */
	@Override
	public int getSize() {
		return playList.size();
	}

	/**
	 * Gets the PlayListEntry at the specified index.
	 */
	@Override
	public PlayListEntry getElementAt(int index) {
		return playList.getEntry(index);
	}
	/**
	 * Adds files to the play list.
	 * @param files The files to add.
	 */
	public void addFiles(File[] files){
		playList.addEntries(files);
		int lastEntry = playList.size() - 1;
		this.fireContentsChanged(this, lastEntry - files.length, lastEntry);		
	}
	
	/**
	 * Updates the last entry
	 */
	public void addedLastEntry(){
		int lastEntry = playList.size() - 1;
		this.fireContentsChanged(this, lastEntry, lastEntry);
	}
	
	/**
	 * Updates the list if an entry has been removed.
	 * @param index The index of the entry which has been removed.
	 */
	public void removedEntry(int index){
		this.fireIntervalRemoved(this, index, index);
	}
	
	/**
	 * Sorts the list with the specified sort method.
	 * @param sortMethod The sort method to use for sorting.
	 */
	public void sort(SortMethod sortMethod){
		this.playList.sort(sortMethod);
		int lastEntry = playList.size() - 1;
		this.fireContentsChanged(this, 0, lastEntry);
	}
	
	/**
	 * Loads the play list and updates the list.
	 * @param file The play list file.
	 * @throws JAXBException
	 */
	public void load(File file) throws JAXBException{
		this.playList = PlayList.playListLoad(file);
		int lastEntry = playList.size() - 1;
		this.fireContentsChanged(this, 0, lastEntry);
	}
	
	/**
	 * Saves the current play list.
	 * @throws FileNotFoundException
	 * @throws JAXBException
	 */	
	public void save() throws FileNotFoundException, JAXBException{
		this.playList.playListSave();
	}
	
	/**
	 * Saves the current play list under a new name.
	 * @param file The file under which the play list should be saved.
	 * @throws FileNotFoundException
	 * @throws JAXBException
	 */
	public void saveAs(File file) throws FileNotFoundException, JAXBException{
		this.playList.setPath(file.getAbsolutePath());
		save();
	}
	
	/**
	 * Gets the path to the play list file.
	 * @return The path to the play list file.
	 */
	public String getPath(){
		return this.playList.getPath();
	}
	/**
	 * Sets the path to the play list file.
	 * @param path The path to the play list file.
	 */
	public void setPath(String path){
		this.playList.setPath(path);
	}
}
