package player;

import generic.Helper;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.filechooser.FileFilter;
import javax.xml.bind.JAXBException;

import player.PlayListEntry.SortMethod;

import configuration.Control;
import configuration.ImageControl;
import configuration.Skin;

/**
 * The class PlayListWindow inherits from JFrame and presents the play list to the user.
 * @author Martin Henke
 *
 */
public class PlayListWindow extends JFrame implements MouseListener, ActionListener {
	private static final long serialVersionUID = 960729580035112993L;
	
	private JPanel contentPane;
	private JScrollPane scrollPane;
	private JList<PlayListEntry> list;
	private MainDialog playerWindow;
	private PlayListListModel model;
	private int playIndex = -1;
	private JPopupMenu popupMenu;
	private JMenu mnSort;
	private JMenuItem mntmPath_A_Z;
	private JMenuItem mntmPath_Z_A;
	private JMenuItem mntmName_A_Z;
	private JMenuItem mntmName_Z_A;
	private JMenuItem mntmSize_0_9;
	private JMenuItem mntmSize_9_0;
	private JMenu mnAdd;
	private JMenuItem mntmAddToPlay;
	private JFileChooser playListFileChooser;
	
	/**
	 * Initializes the window and provides a reference to the player window.
	 * @param playerWindow The player Window which controls the program.
	 */
	public PlayListWindow(MainDialog playerWindow) {
		this.playerWindow = playerWindow;
		setTitle("Play List");
		setType(Type.UTILITY);
		setBounds(100, 100, 450, 300);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);
		
		scrollPane = new JScrollPane();
		contentPane.add(scrollPane, BorderLayout.CENTER);
		
		list = new JList<PlayListEntry>();
		list.addMouseListener(this);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		scrollPane.setViewportView(list);
		
		popupMenu = new JPopupMenu();
		addPopup(list, popupMenu);
		
		mnSort = new JMenu("Sort");
		popupMenu.add(mnSort);
		
		mntmPath_A_Z = new JMenuItem("Path A...Z");
		mntmPath_A_Z.addActionListener(this);
		mnSort.add(mntmPath_A_Z);
		
		mntmPath_Z_A = new JMenuItem("Path Z...A");
		mntmPath_Z_A.addActionListener(this);
		mnSort.add(mntmPath_Z_A);
		
		mntmName_A_Z = new JMenuItem("Name A...Z");
		mntmName_A_Z.addActionListener(this);
		mnSort.add(mntmName_A_Z);
		
		mntmName_Z_A = new JMenuItem("Name Z...A");
		mntmName_Z_A.addActionListener(this);
		mnSort.add(mntmName_Z_A);
		
		mntmSize_0_9 = new JMenuItem("Size small to large");
		mntmSize_0_9.addActionListener(this);
		mnSort.add(mntmSize_0_9);
		
		mntmSize_9_0 = new JMenuItem("Size large to small");
		mntmSize_9_0.addActionListener(this);
		mnSort.add(mntmSize_9_0);
		
		mnAdd = new JMenu("Add");
		popupMenu.add(mnAdd);
		
		mntmAddToPlay = new JMenuItem("Add to Play List");
		mntmAddToPlay.addActionListener(this);
		mnAdd.add(mntmAddToPlay);
		loadDefaultList();
		
		playListFileChooser = new JFileChooser();
		playListFileChooser.setAcceptAllFileFilterUsed(false);
		playListFileChooser.addChoosableFileFilter(new FileFilter(){
			@Override
			public boolean accept(File f) {
				return (f.isDirectory() || f.getName().endsWith(PlayList.EXTENSION));
			}

			@Override
			public String getDescription() {
				return "PlayList (*" + PlayList.EXTENSION + ")";
			}
			
		});
		playListFileChooser.setMultiSelectionEnabled(false);
	}

	/**
	 * Handles mouse clicks.
	 * @param e The associated MouseEvent object.
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.getSource() == list) {
			listMouseClicked(e);
		}
	}
	/**
	 * Currently not used
	 */
	public void mouseEntered(MouseEvent e) { }
	/**
	 * Currently not used
	 */
	public void mouseExited(MouseEvent e) { }
	/**
	 * Currently not used
	 */
	public void mousePressed(MouseEvent e) { }
	/**
	 * Currently not used
	 */
	public void mouseReleased(MouseEvent e) { }

	/**
	 * Handles clicks on the list of play list entries. 
	 * @param e The associated MouseEvent object.
	 */
	protected void listMouseClicked(MouseEvent e) {
		if (e.getClickCount() > 1){
			listMouseDoubleClicked(e);
		}
	}
	
	/**
	 * Handles double clicks on the list of play list entries. Calls playSelectedFile().
	 * @param e The associated MouseEvent object.
	 */
	private void listMouseDoubleClicked(MouseEvent e){
		playSelectedFile();
	}
	
	/**
	 * Adds files to the play list.
	 * @param mediaFiles The files to add.
	 */
	public void addFiles(File[] mediaFiles){
		model.addFiles(mediaFiles);
	}
	
	/**
	 * Plays the selected file
	 */
	private void playSelectedFile(){
		playFile(model.getElementAt(list.getSelectedIndex()));
	}
	
	/**
	 * Plays the specified play list entry.
	 * @param entry The entry to play.
	 */
	private void playFile(PlayListEntry entry){
		if (entry != null){
			playIndex = list.getSelectedIndex();
			playerWindow.loadMediaFile(new File(entry.getFullPath()));
		}		
	}
	
	/**
	 * Plays the next file in the list.
	 */
	public void playNextFile(){
		if (model.getSize() - 1 > playIndex){
			playIndex++;
			list.setSelectedIndex(playIndex);
			playSelectedFile();
		}
	}
	
	/**
	 * Sets the skin settings to the window.
	 * @param skin The skin to apply.
	 * @param setWindowLocation Defines if the window location should be set.
	 * @param parentBounds The bounds of the player window.
	 */
	public void setSkin(Skin skin, boolean setWindowLocation, Rectangle parentBounds){
		setSkinWindow(skin.getWindow(), setWindowLocation, parentBounds);
		if (skin.getPlayList().isVisible()){									// If play list button is used in the player window...
			this.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);		// ...it is allowed to close this window.
		} else {																// If no play list button is used in the player window...
			this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);	// ... this window MUST NOT be closed (as the user could not select songs).
		}
		Control menu = skin.getMenu();		
		MP3Player.setSkinMenu(this.popupMenu, menu.getBackColor().toColor(), menu.getForeColor().toColor(), menu.getFont().toFont());
	}
	/**
	 * Applies the skin settings to the window.
	 * @param window The window settings.
	 * @param setWindowLocation Defines if the window location should be set.
	 * @param parentBounds The bounds of the media player window.
	 */
	private void setSkinWindow(ImageControl window, boolean setWindowLocation, Rectangle parentBounds){
		this.setBackground(window.getBackColor().toColor());
		this.setForeground(window.getForeColor().toColor());
		this.setFont(window.getFont().toFont());
		list.setBackground(this.getBackground());
		list.setForeground(this.getForeground());
		list.setFont(this.getFont());
		if (setWindowLocation){
			this.setLocation(parentBounds.x, parentBounds.y + parentBounds.height);
		}
	}
	
	/**
	 * Gets the number of play list entries.
	 * @return The number of play list entries.
	 */
	public int getEntryCount() { return model.getSize(); }
	
	/**
	 * Plays the file at the specified location in the play list. 
	 * @param index The index of the play list entry to play.
	 */
	public void playFile(int index){
		if (index < model.getSize()){
			list.setSelectedValue(model.getElementAt(index), true);
			playSelectedFile();
		}
	}
	
	/**
	 * Handles the popup menu.
	 * @param component The component holding the popup menu.
	 * @param popup The popup menu to control.
	 */
	private static void addPopup(Component component, final JPopupMenu popup) {
		component.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showMenu(e);
				}
			}
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showMenu(e);
				}
			}
			private void showMenu(MouseEvent e) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		});
	}
	
	/**
	 * Handles clicks.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == mntmAddToPlay) {
			mntmAddToPlayActionPerformed(e);
		} else if (e.getSource() == mntmPath_A_Z) {
			mntmPath_A_ZActionPerformed(e);
		} else if (e.getSource() == mntmPath_Z_A) {
			mntmPath_Z_AActionPerformed(e);
		} else if (e.getSource() == mntmName_A_Z) {
			mntmName_A_ZActionPerformed(e);
		} else if (e.getSource() == mntmName_Z_A) {
			mntmName_Z_AActionPerformed(e);
		} else if (e.getSource() == mntmSize_0_9) {
			mntmSize_0_9ActionPerformed(e);
		} else if (e.getSource() == mntmSize_9_0) {
			mntmSize_9_0ActionPerformed(e);
		}
	}
	
	/**
	 * Sorts by file name from A...Z.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmName_A_ZActionPerformed(ActionEvent e) {
		sort(SortMethod.Name_A_Z);
	}
	/**
	 * Sorts by file name from Z...A.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmName_Z_AActionPerformed(ActionEvent e) {
		sort(SortMethod.Name_Z_A);
	}
	/**
	 * Sorts by path from A...Z.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmPath_A_ZActionPerformed(ActionEvent e) {
		sort(SortMethod.FullPath_A_Z);
	}
	/**
	 * Sorts by path from Z...A.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmPath_Z_AActionPerformed(ActionEvent e) {
		sort(SortMethod.FullPath_Z_A);
	}
	/**
	 * Sorts by file size from small to large. 
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmSize_0_9ActionPerformed(ActionEvent e) {
		sort(SortMethod.FileSize_0_9);
	}
	/**
	 * Sorts by file size from large to small. 
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmSize_9_0ActionPerformed(ActionEvent e) {
		sort(SortMethod.FileSize_9_0);
	}
	
	/**
	 * Sorts with the specified sort method.
	 * @param sortMethod The sort method to use for sorting.
	 */
	private void sort(SortMethod sortMethod){
		model.sort(sortMethod);
	}	
	
	/**
	 * Adds files to the play list.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmAddToPlayActionPerformed(ActionEvent e) {
		playerWindow.addMedia();
	}

	/**
	 * Creates a new play list.
	 */
	public void newPlayList(){
		model = new PlayListListModel();
		this.list.setModel(model);
	}
	
	/**
	 * Loads the default play list.
	 */
	private void loadDefaultList(){
		File defaultPath = new File(Helper.getJarDirectory(), PlayList.DEFAULT_NAME);
		try {
			loadPlayList(defaultPath);
		} catch (FileNotFoundException | JAXBException e) {
			model = new PlayListListModel();
			model.setPath(defaultPath.getAbsolutePath());
			this.list.setModel(model);
		}
	}
	
	/**
	 * Loads the specified play list.
	 * @param playListFile The play list file to load.
	 * @throws FileNotFoundException
	 * @throws JAXBException
	 */
	private void loadPlayList(File playListFile) throws FileNotFoundException, JAXBException{
		if (playListFile.isDirectory() || !playListFile.exists()){
			throw new java.io.FileNotFoundException();
		}
		model = new PlayListListModel();
		model.load(playListFile);
		this.list.setModel(model);
	}
	
	/**
	 * Opens a JFileChooser to load a play list.
	 */
	public void loadPlayList(){
		playListFileChooser.setDialogTitle("Select Play List");
		if (playListFileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION){
			try {
				loadPlayList(playListFileChooser.getSelectedFile());
			} catch (JAXBException | FileNotFoundException e) {
				JOptionPane.showMessageDialog(this, "Unable to load play list.", "Play List Error", JOptionPane.WARNING_MESSAGE);
			}
		}
	}
	
	/**
	 * Saves a play list
	 * @param saveAs Defines if the SaveFile dialog should be shown.
	 */
	public void savePlayList(boolean saveAs){		
		if (!saveAs && (model.getPath() == null || model.getPath().length() == 0)){
			saveAs = true;
		}
		if (saveAs){
			playListFileChooser.setDialogTitle("Set a Play List name");
			if (playListFileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION){
				try {
					model.saveAs(playListFileChooser.getSelectedFile());
				} catch (FileNotFoundException | JAXBException e) {
					JOptionPane.showMessageDialog(this, "Unable to save play list.", "Play List Error", JOptionPane.WARNING_MESSAGE);
				}
			}
		} else {
			try {
				model.save();
			} catch (FileNotFoundException | JAXBException e) {
				JOptionPane.showMessageDialog(this, "Unable to save play list.", "Play List Error", JOptionPane.WARNING_MESSAGE);
			}
		}
	}
}
