package editor;

import configuration.Control;

/**
 * The class ControlTableModel inherits from EmptyTableModel and is used to show the settings of a Control object.
 * @author Martin Henke
 *
 */
public class ControlTableModel extends EmptyTableModel{

	private static final long serialVersionUID = 680217765362332441L;

	/**
	 * The enum ControlValueTypes defines the possible settings for Control objects.
	 */
	public static enum ControlValueTypes {
		NONE, Location, Size, BackColor, ForeColor, Font, Visible;

		public static ControlValueTypes fromInteger(int value){
			switch (value){
			case 0:
				return NONE;
			case 1:
				return Location;
			case 2:
				return Size;
			case 3:
				return BackColor;
			case 4:
				return ForeColor;
			case 5:
				return Font;
			case 6:
				return Visible;
			default:
				return null;
			}
		}
	};
	private Control control;
	
	/**
	 * Sets the Control object to provide the settings.
	 * @param control The Control object to provide the settings. 
	 */
	public void setData(Control control){
		this.control = control;
	}
	
	/**
	 * Returns the number of settings (always 6).
	 */
	@Override
	public int getRowCount() {
		return 6;
	}

	/**
	 * Returns the data to show in the table.
	 * @param rowIndex The row in which the data are located.
	 * @param columnIndex The columnIndex which specifies the type of data. columnIndex=0 defines the object name, columnIndex=1 defines the setting.
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex){
		case 0:
			return getObjectName(ControlValueTypes.fromInteger(rowIndex + 1));
		default:
			return getObjectValue(ControlValueTypes.fromInteger(rowIndex + 1));
		}
	}
	
	/**
	 * Sets the value in the table. Editing can only take place if columnIndex=1 and is only allowed for strings.
	 */
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex){
		if (columnIndex == 1){
			ControlValueTypes valueType = ControlValueTypes.fromInteger(rowIndex + 1);		
			switch (valueType){
			case Location: 	// No editing in table
				break;
			case Size: 		// No editing in table
				break;
			case BackColor:	// No editing in table
				break;
			case ForeColor:	// No editing in table
				break;
			case Font:		// No editing in table
				break;
			case Visible:	// No editing in table
				break;
			case NONE:
			default:		// Illegal value
				break;
			}		
		}
	}	
	
	/**
	 * Returns if a cell is editable. Only cells with columnIndex=1 can be edited if they hold a String setting.
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex){
		if (columnIndex != 1){
			return false;
		} else {
			ControlValueTypes valueType = ControlValueTypes.fromInteger(rowIndex + 1);
			switch (valueType){
			case Location:
				return false;
			case Size:
				return false;
			case BackColor:
				return false;
			case ForeColor:
				return false;
			case Font:
				return false;
			case Visible:
				return false;
			case NONE:
			default:
				return false;
			
			}
		}
	}
	
	/**
	 * Gets the name of a setting.
	 * @param valueType The enum value to get the setting name for.
	 * @return The name of the setting.
	 */
	private String getObjectName(ControlValueTypes valueType){
		switch (valueType){
		case Location:
			return "Location";
		case Size:
			return "Size";
		case BackColor:
			return "BackColor";
		case ForeColor:
			return "ForeColor";
		case Font:
			return "Font";
		case Visible:
			return "Visible";
		default:
			return null;
		}
	}
	
	/**
	 * Gets the String representation of a setting.
	 * @param valueType The enum value to get the setting value for.
	 * @return The String representation of the setting. 
	 */
	private String getObjectValue(ControlValueTypes valueType){
		if (control != null){
			switch (valueType){
			case Location:
				return Formatter.format(control.getLocation());
			case Size:
				return Formatter.format(control.getSize());
			case BackColor:
				return Formatter.format(control.getBackColor());
			case ForeColor:
				return Formatter.format(control.getForeColor());
			case Font:
				return Formatter.format(control.getFont());
			case Visible:
				return Formatter.format(control.isVisible());
			default:
				return null;
			}
		} else {
			return null;
		}
	}
}
