package editor;

import configuration.ImageControl;

/**
 * The class ImageControlTableModel inherits from EmptyTableModel and is used to show the settings of a ImageControl object.
 * @author Martin Henke
 *
 */
public class ImageControlTableModel extends EmptyTableModel {
	
	private static final long serialVersionUID = 7407989728499144870L;
	
	/**
	 * The enum ImageControlValueTypes defines the possible settings for ImageControl objects.
	 */
	public static enum ImageControlValueTypes {
		NONE, Location, Size, BackColor, ForeColor, Font, Text, Visible, Image;

		public static ImageControlValueTypes fromInteger(int value){
			switch (value){
			case 0:
				return NONE;
			case 1:
				return Location;
			case 2:
				return Size;
			case 3:
				return BackColor;
			case 4:
				return ForeColor;
			case 5:
				return Font;
			case 6:
				return Text;
			case 7:
				return Visible;
			case 8:
				return Image;
			default:
				return null;
			}
		}
	};

	private ImageControl imageControl;
	
	/**
	 * Sets the ImageControl object to provide the settings.
	 * @param imageControl The ImageControl object to provide the settings. 
	 */
	public void setData(ImageControl imageControl){
		this.imageControl = imageControl;
	}
	
	/**
	 * Returns the number of settings (always 8).
	 */
	@Override
	public int getRowCount() {
		return 8;
	}

	/**
	 * Returns the data to show in the table.
	 * @param rowIndex The row in which the data are located.
	 * @param columnIndex The columnIndex which specifies the type of data. columnIndex=0 defines the object name, columnIndex=1 defines the setting.
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex){
		case 0:
			return getObjectName(ImageControlValueTypes.fromInteger(rowIndex + 1));
		default:
			return getObjectValue(ImageControlValueTypes.fromInteger(rowIndex + 1));
		}
	}
	
	/**
	 * Sets the value in the table. Editing can only take place if columnIndex=1 and is only allowed for strings.
	 */
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex){
		if (columnIndex == 1){
			ImageControlValueTypes valueType = ImageControlValueTypes.fromInteger(rowIndex + 1);		
			switch (valueType){
			case Location: 	// No editing in table
				break;
			case Size: 		// No editing in table
				break;
			case BackColor:	// No editing in table
				break;
			case ForeColor:	// No editing in table
				break;
			case Font:		// No editing in table
				break;
			case Text:		// No editing in table
				imageControl.setText((String) aValue);
				break;
			case Visible:	// No editing in table
				break;
			case Image:		// No editing in table
				break;
			case NONE:
			default:		// Illegal value
				break;
			}		
		}
	}	
	
	/**
	 * Returns if a cell is editable. Only cells with columnIndex=1 can be edited if they hold a String setting.
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex){
		if (columnIndex != 1){
			return false;
		} else {
			ImageControlValueTypes valueType = ImageControlValueTypes.fromInteger(rowIndex + 1);
			switch (valueType){
			case Location:
				return false;
			case Size:
				return false;
			case BackColor:
				return false;
			case ForeColor:
				return false;
			case Font:
				return false;
			case Text:
				return true;
			case Visible:
				return false;
			case Image:
				return false;
			case NONE:
			default:
				return false;
			
			}
		}
	}
	
	/**
	 * Gets the name of a setting.
	 * @param valueType The enum value to get the setting name for.
	 * @return The name of the setting.
	 */
	private String getObjectName(ImageControlValueTypes valueType){
		switch (valueType){
		case Location:
			return "Location";
		case Size:
			return "Size";
		case BackColor:
			return "BackColor";
		case ForeColor:
			return "ForeColor";
		case Font:
			return "Font";
		case Text:
			return "Text";
		case Visible:
			return "Visible";
		case Image:
			return "Image";
		default:
			return null;
		}
	}
	
	/**
	 * Gets the String representation of a setting.
	 * @param valueType The enum value to get the setting value for.
	 * @return The String representation of the setting. 
	 */
	private String getObjectValue(ImageControlValueTypes valueType){
		if (imageControl != null){
			switch (valueType){
			case Location:
				return Formatter.format(imageControl.getLocation());
			case Size:
				return Formatter.format(imageControl.getSize());
			case BackColor:
				return Formatter.format(imageControl.getBackColor());
			case ForeColor:
				return Formatter.format(imageControl.getForeColor());
			case Font:
				return Formatter.format(imageControl.getFont());
			case Text:
				return imageControl.getText();
			case Visible:
				return Formatter.format(imageControl.isVisible());
			case Image:			
				return Formatter.format(imageControl.getImage());
			default:
				return null;
			}
		} else {
			return null;
		}
	}
}
