package editor;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import configuration.Control;
import configuration.TextControl;
import configuration.Button;
import configuration.ImageControl;
import configuration.Skin;
import editor.ButtonTableModel.ButtonValueTypes;
import editor.ControlTableModel.ControlValueTypes;
import editor.TextControlTableModel.TextControlValueTypes;
import editor.ImageControlTableModel.ImageControlValueTypes;

import generic.*;
import panels.BoolPanel;
import panels.PointPanel;
import panels.DimensionPanel;
import panels.ColorPanel;
import panels.FontPanel;
import panels.ImagePanel;

/**
 * The class MainWindow displays the main functions of the skin editor.
 * @author Martin Henke
 *
 */
public class MainWindow extends JFrame implements ActionListener, ListSelectionListener {

	/**
	 * The enum ListEntryConstants contains constants which match the settings in the <code>Skin</code> class.
	 *
	 */
	static enum ListEntryConstants
	{
		Window, Play, Pause, Stop, FastForward, Rewind, Skip, Restart, PlayList, Menu, Title, ElapsedTime;
		
		public static ListEntryConstants fromInteger(int value){
			switch (value){
			case 0:
				return Window;
			case 1:
				return Play;
			case 2:
				return Pause;
			case 3:
				return Stop;
			case 4:
				return FastForward;
			case 5:
				return Rewind;
			case 6:
				return Skip;
			case 7:
				return Restart;
			case 8:
				return PlayList;
			case 9:
				return Menu;
			case 10:
				return Title;
			case 11:
				return ElapsedTime;
			default:
				return null;
			}
		}
	};
	
	/**
	 * The enum PanelTypes defines the input panels which can be used. There are panels for each setting except for String settings as they can be edited
	 * directly in the table. 
	 */
	private enum PanelTypes{
		PointPanel(0), DimensionsPanel(1), ColorPanel(2), FontPanel(3), ImagePanel(4), BoolPanel(5);

		private final int index;
				
		private PanelTypes(int index){
			this.index = index;
		}
		
		/**
		 * Returns the index for the panel.
		 * @return The index for the panel.
		 */
		public int getIndex(){ return index; } 
	};
	
	private static final long serialVersionUID = 1723486534890358099L;
	
	private ListEntryConstants selectedEntry = null;
	private Button currentButton = null;
	private Control currentControl = null;
	private TextControl currentTextControl = null;
	private ImageControl currentImageControl = null;
	
	private ImageControlValueTypes imageControlValueType = ImageControlValueTypes.Location;
	private ControlValueTypes controlValueType = ControlValueTypes.Location;
	private TextControlValueTypes textControlValueType = TextControlValueTypes.Location;
	private ButtonValueTypes buttonValueType = ButtonValueTypes.Location;

	private JPanel contentPane;
	private JMenuBar menuBar;
	private JMenu mnFile;
	private JMenuItem mntmOpen;
	private JMenuItem mntmSave;
	private JSeparator separator;
	private JMenuItem mntmExit;
	private JMenuItem mntmSaveAs;
	private JPanel panel;
	private JList<String> lstSettings;
	private JMenuItem mntmNew;
	private configuration.Skin skin;
	private JSeparator separator_1;
	private JTable tblSettings;
	private JScrollPane scrollPane;
	private JPanel cardPanel;
	private PointPanel pointPanel;
	private DimensionPanel dimensionPanel;
	private ColorPanel colorPanel;
	private FontPanel fontPanel;
	private ImagePanel imagePanel;
	private BoolPanel boolPanel;
	private JPanel pnlButtons;
	private JButton btnAccept;
	private JMenu mnHelp;
	private JMenuItem mntmInfo;
		
	public MainWindow() {
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 800, 460);
		
		menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		mnFile = new JMenu("File");
		mnFile.setMnemonic('f');
		menuBar.add(mnFile);
		
		mntmNew = new JMenuItem("New");
		mntmNew.addActionListener(this);
		mntmNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		mntmNew.setMnemonic(KeyEvent.VK_N);
		mnFile.add(mntmNew);
		
		mntmOpen = new JMenuItem("Open");
		mntmOpen.addActionListener(this);
		mntmOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		mntmOpen.setMnemonic(KeyEvent.VK_O);
		mnFile.add(mntmOpen);
		
		separator = new JSeparator();
		mnFile.add(separator);
		
		mntmSave = new JMenuItem("Save");
		mntmSave.addActionListener(this);
		mntmSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		mntmSave.setMnemonic(KeyEvent.VK_S);
		mnFile.add(mntmSave);
		
		mntmSaveAs = new JMenuItem("Save As...");
		mntmSaveAs.addActionListener(this);
		mntmSaveAs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		mntmSaveAs.setMnemonic(KeyEvent.VK_A);
		mnFile.add(mntmSaveAs);
		
		separator_1 = new JSeparator();
		mnFile.add(separator_1);
		
		mntmExit = new JMenuItem("Exit");
		mntmExit.addActionListener(this);
		mntmExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.ALT_MASK));
		mntmExit.setMnemonic(KeyEvent.VK_X);
		mnFile.add(mntmExit);
		
		mnHelp = new JMenu("Help");
		mnHelp.setMnemonic(KeyEvent.VK_H);
		menuBar.add(mnHelp);
		
		mntmInfo = new JMenuItem("Info");
		mntmInfo.addActionListener(this);
		mntmInfo.setMnemonic(KeyEvent.VK_I);
		mnHelp.add(mntmInfo);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);
		
		panel = new JPanel();
		contentPane.add(panel, BorderLayout.CENTER);
		panel.setLayout(null);
		
		lstSettings = new JList<String>();
		lstSettings.setFont(new Font("Tahoma", Font.PLAIN, 11));
		lstSettings.addListSelectionListener(this);
		lstSettings.setModel(new AbstractListModel<String>() {
			private static final long serialVersionUID = 1352327712220734305L;
			String[] values = new String[] {"Window", "Play", "Pause", "Stop", "Fast Forward", "Rewind", "Skip", "Restart", "Playlist", "Menu", "Title", "Elapsed Time"};
			public int getSize() {
				return values.length;
			}
			public String getElementAt(int index) {
				return values[index];
			}
		});
		lstSettings.setBounds(10, 11, 115, 346);
		panel.add(lstSettings);
		
		cardPanel = new JPanel();
		cardPanel.setVisible(false);
		cardPanel.setBounds(486, 10, 292, 347);
		panel.add(cardPanel);
		cardPanel.setLayout(new CardLayout(0, 0));
		
		pointPanel = new PointPanel();
		cardPanel.add(pointPanel, "name_23751746654463");
		
		dimensionPanel = new DimensionPanel();
		cardPanel.add(dimensionPanel, "name_23905945882088");
		
		colorPanel = new ColorPanel();
		cardPanel.add(colorPanel, "name_23924486320398");
		
		fontPanel = new FontPanel();
		cardPanel.add(fontPanel, "name_23936632346429");
		
		imagePanel = new ImagePanel();
		cardPanel.add(imagePanel, "name_23941080325572");
		
		boolPanel = new BoolPanel();
		cardPanel.add(boolPanel, "name_boolPanel_1");
		
		scrollPane = new JScrollPane();
		scrollPane.setBounds(135, 10, 341, 347);
		panel.add(scrollPane);
		
		tblSettings = new JTable();
		tblSettings.setFont(new Font("Tahoma", Font.PLAIN, 11));
		tblSettings.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		tblSettings.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
            	if (!e.getValueIsAdjusting()){
            		tblSettingsValueChanged(e);
            	}
            }
		});
		tblSettings.setModel(new EmptyTableModel());
		tblSettings.getSelectionModel().addListSelectionListener(this);
		
		scrollPane.setViewportView(tblSettings);
		
		pnlButtons = new JPanel();
		pnlButtons.setBounds(new Rectangle(0, 0, 0, 23));
		contentPane.add(pnlButtons, BorderLayout.SOUTH);
		pnlButtons.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
		
		btnAccept = new JButton("Accept");
		btnAccept.addActionListener(this);
		btnAccept.setEnabled(false);
		pnlButtons.add(btnAccept);
		createNewSkin();
	}

	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == mntmInfo) {
			mntmInfoActionPerformed(e);
		} else if (e.getSource() == btnAccept) {
			btnAcceptActionPerformed(e);
		} else if (e.getSource() == mntmSave) {
			mntmSaveActionPerformed(e);
		} else if (e.getSource() == mntmSaveAs) {
			mntmSaveAsActionPerformed(e);
		} else if (e.getSource() == mntmExit) {
			mntmExitActionPerformed(e);
		} else if (e.getSource() == mntmOpen) {
			mntmOpenActionPerformed(e);
		} else if (e.getSource() == mntmNew) {
			mntmNewActionPerformed(e);
		}
	}
	
	/**
	 * Calls createNewSkin(). 
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmNewActionPerformed(ActionEvent e) {
		createNewSkin();
	}
	/**
	 * Calls skinLoad().
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmOpenActionPerformed(ActionEvent e) {
		skinLoad();
	}
	/**
	 * Closes the program.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmExitActionPerformed(ActionEvent e) {
		System.exit(0);
	}
	/**
	 * Calls skinSave().
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmSaveActionPerformed(ActionEvent e) {
		skinSave();
	}
	/**
	 * Calls skinSaveAs().
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmSaveAsActionPerformed(ActionEvent e) {
		skinSaveAs();
	}
	/**
	 * Calls accept().
	 * @param e The associated ActionEvent object.
	 */
	protected void btnAcceptActionPerformed(ActionEvent e) {
		accept();
	}
	/**
	 * Makes the change to the table model and updates the table. 
	 */
	private void accept(){
		int row = this.tblSettings.getSelectedRow();
		if (row > -1){
			if (currentButton != null){
				saveButtonEntryToSkin();
				ButtonTableModel btm = (ButtonTableModel) this.tblSettings.getModel();
				btm.fireTableCellUpdated(row, 1);
			} else if (currentControl != null){
				saveControlEntryToSkin();
				ControlTableModel ctm = (ControlTableModel) this.tblSettings.getModel();
				ctm.fireTableCellUpdated(row, 1);
			} else if (currentTextControl != null){
				saveTextControlEntryToSkin();
				TextControlTableModel tctm =(TextControlTableModel) this.tblSettings.getModel();
				tctm.fireTableCellUpdated(row, 1);
			} else if (currentImageControl != null){
				saveImageControlEntryToSkin();
				ImageControlTableModel ctm = (ImageControlTableModel) this.tblSettings.getModel();
				ctm.fireTableCellUpdated(row, 1);
			}
		}
	}
	
	/**
	 * Handles list selection changes.
	 * @param e The associated ListSelectionEvent object.
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource() == lstSettings) {
			lstSettingsValueChanged(e);
		} 
	}
	/**
	 * Displays the settings for the selected object in the table
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void lstSettingsValueChanged(ListSelectionEvent e) {
		saveSelectedEntryToSkin();
		selectedEntry = ListEntryConstants.fromInteger(lstSettings.getSelectedIndex());
		currentButton = null;
		currentControl = null;
		currentTextControl = null;
		currentImageControl = null;
		if (selectedEntry != null){			
			switch (selectedEntry){
			case Window:
				currentImageControl = skin.getWindow();
				break;
			case Play:
				currentButton = skin.getPlay();
				break;
			case Pause:
				currentButton = skin.getPause();
				break;
			case Stop:
				currentButton = skin.getStop();
				break;
			case FastForward:
				currentButton = skin.getFastForward();
				break;
			case Rewind:
				currentButton = skin.getRewind();
				break;
			case Skip:
				currentButton = skin.getSkip();
				break;
			case Restart:
				currentButton = skin.getRestart();
				break;
			case PlayList:
				currentButton = skin.getPlayList();
				break;
			case Menu:
				currentControl = skin.getMenu();
				break;
			case Title:
				currentControl = skin.getTitle();
				break;
			case ElapsedTime:
				currentControl = skin.getElapsedTime();
			default:
				break;
			}
			
			if (currentButton != null){
				ButtonTableModel buttonModel = new ButtonTableModel();
				buttonModel.setData(currentButton);
				this.tblSettings.setModel(buttonModel);
				this.tblSettings.setRowSelectionInterval(0, 0);
				
			} 	else if (currentControl != null){
				ControlTableModel controlModel = new ControlTableModel();
				controlModel.setData(currentControl);
				this.tblSettings.setModel(controlModel);
				this.tblSettings.setRowSelectionInterval(0, 0);
				
			} else if (currentTextControl != null){
				TextControlTableModel textControlModel = new TextControlTableModel();
				textControlModel.setData(currentTextControl);
				this.tblSettings.setModel(textControlModel);
				this.tblSettings.setRowSelectionInterval(0, 0);
				
			} else if (currentImageControl != null){
				ImageControlTableModel imageControlModel = new ImageControlTableModel();
				imageControlModel.setData(currentImageControl);
				this.tblSettings.setModel(imageControlModel);
				this.tblSettings.setRowSelectionInterval(0, 0);
			}
		} else {
			this.btnAccept.setEnabled(false);
			this.cardPanel.setVisible(false);
			this.tblSettings.setModel(new EmptyTableModel());
		}
	}
	/**
	 * Displays the associated panel and copies the data of the selected setting into it.
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void tblSettingsValueChanged(ListSelectionEvent e) {
		int rowIndex = this.tblSettings.getSelectedRow();
		boolean enable = (rowIndex > -1);
		if (enable){
			if (currentButton != null){
				buttonValueType = ButtonValueTypes.fromInteger(rowIndex + 1);
				switch (buttonValueType){
				case Location:
					pointPanel.setValue(currentButton.getLocation());
					cardPanelGoToIndex(PanelTypes.PointPanel);
					break;
				case Size:
					dimensionPanel.setValue(currentButton.getSize());
					cardPanelGoToIndex(PanelTypes.DimensionsPanel);	
					break;
				case BackColor:
					colorPanel.setValue(currentButton.getBackColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case ForeColor:
					colorPanel.setValue(currentButton.getForeColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case Font:
					fontPanel.setValue(currentButton.getFont());
					cardPanelGoToIndex(PanelTypes.FontPanel);
					break;
				case Text:
					cardPanel.setVisible(false);
					btnAccept.setEnabled(false);
					break;
				case Visible:
					boolPanel.setValue(currentButton.isVisible());
					cardPanelGoToIndex(PanelTypes.BoolPanel);
					break;
				case Image:
					imagePanel.setValue(currentButton.getImage());
					cardPanelGoToIndex(PanelTypes.ImagePanel);
					break;
				case Mask:
					imagePanel.setValue(currentButton.getMask());
					cardPanelGoToIndex(PanelTypes.ImagePanel);
					break;
				default:
					break;
				}
			} else if (currentControl != null){
				controlValueType = ControlValueTypes.fromInteger(rowIndex + 1);
				switch(controlValueType){
				case Location:
					pointPanel.setValue(currentControl.getLocation());
					cardPanelGoToIndex(PanelTypes.PointPanel);
					break;
				case Size:
					dimensionPanel.setValue(currentControl.getSize());
					cardPanelGoToIndex(PanelTypes.DimensionsPanel);	
					break;
				case BackColor:
					colorPanel.setValue(currentControl.getBackColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case ForeColor:
					colorPanel.setValue(currentControl.getForeColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case Font:
					fontPanel.setValue(currentControl.getFont());
					cardPanelGoToIndex(PanelTypes.FontPanel);
					break;
				case Visible:
					boolPanel.setValue(currentControl.isVisible());
					cardPanelGoToIndex(PanelTypes.BoolPanel);
					break;
				default:
					break;
				}
			} else if (currentImageControl != null){
				imageControlValueType = ImageControlValueTypes.fromInteger(rowIndex + 1);
				switch (imageControlValueType){
				case Location:
					pointPanel.setValue(currentImageControl.getLocation());
					cardPanelGoToIndex(PanelTypes.PointPanel);
					break;
				case Size:
					dimensionPanel.setValue(currentImageControl.getSize());
					cardPanelGoToIndex(PanelTypes.DimensionsPanel);	
					break;
				case BackColor:
					colorPanel.setValue(currentImageControl.getBackColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case ForeColor:
					colorPanel.setValue(currentImageControl.getForeColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case Font:
					fontPanel.setValue(currentImageControl.getFont());
					cardPanelGoToIndex(PanelTypes.FontPanel);
					break;
				case Text:
					cardPanel.setVisible(false);
					btnAccept.setEnabled(false);
					break;
				case Visible:
					boolPanel.setValue(currentImageControl.isVisible());
					cardPanelGoToIndex(PanelTypes.BoolPanel);
					break;
				case Image:
					imagePanel.setValue(currentImageControl.getImage());
					cardPanelGoToIndex(PanelTypes.ImagePanel);
					break;
				default:
					break;
				}
			} else if (currentTextControl != null){
				textControlValueType = TextControlValueTypes.fromInteger(rowIndex + 1);
				switch (textControlValueType){
				case Location:
					pointPanel.setValue(currentTextControl.getLocation());
					cardPanelGoToIndex(PanelTypes.PointPanel);
					break;
				case Size:
					dimensionPanel.setValue(currentTextControl.getSize());
					cardPanelGoToIndex(PanelTypes.DimensionsPanel);	
					break;
				case BackColor:
					colorPanel.setValue(currentTextControl.getBackColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case ForeColor:
					colorPanel.setValue(currentTextControl.getForeColor());
					cardPanelGoToIndex(PanelTypes.ColorPanel);
					break;
				case Font:
					fontPanel.setValue(currentTextControl.getFont());
					cardPanelGoToIndex(PanelTypes.FontPanel);
					break;
				case Text:
					cardPanel.setVisible(false);
					btnAccept.setEnabled(false);
					break;
				case Visible:
					boolPanel.setValue(currentTextControl.isVisible());
					cardPanelGoToIndex(PanelTypes.BoolPanel);
					break;
				default:
					break;
				}
			}
		}
		btnAccept.setEnabled(enable);
	}
	
	/**
	 * Saves the selected entry into the Skin object.
	 */
	private void saveSelectedEntryToSkin(){
		if (selectedEntry != null){
			switch(selectedEntry){
			case Window:
				saveImageControlEntryToSkin();
			break;
			case Title:
			case ElapsedTime:
			case Menu:
				saveControlEntryToSkin();
				break;
			default:
				saveButtonEntryToSkin();
				break;
			}
		}
	}
	
	/**
	 * Saves the selected Control entry to the Skin object.
	 */
	private void saveControlEntryToSkin(){
		if (selectedEntry != null){
			switch (selectedEntry){
			case Title:
				saveControlEntryToSkin(skin.getTitle());
				break;
			case ElapsedTime:
				saveControlEntryToSkin(skin.getElapsedTime());
				break;
			case Menu:
				saveControlEntryToSkin(skin.getMenu());
				break;							
			default:
				break;
			}
		}
	}

	/**
	 * Saves the specified Control to the Skin object.
	 * @param control The Control to save to.
	 */
	private void saveControlEntryToSkin(Control control){
		int rowIndex = this.tblSettings.getSelectedRow();
		boolean enable = (rowIndex > -1);
		if (enable){
			if (currentControl != null){
				controlValueType = ControlValueTypes.fromInteger(rowIndex + 1);
				switch (controlValueType){
				case Location:
					control.setLocation(pointPanel.getValue());
					break;
				case Size:
					control.setSize(dimensionPanel.getValue());
					break;
				case BackColor:
					control.setBackColor(colorPanel.getValue());
					break;
				case ForeColor:
					control.setForeColor(colorPanel.getValue());
					break;
				case Font:
					control.setFont(fontPanel.getValue());
					break;
				case Visible:
					control.setVisible(boolPanel.getValue());
					break;
				default:
					break;
				}

			}
		}
	}
	
	/**
	 * Saves the selected ImageControl entry to the Skin object.
	 */
	private void saveImageControlEntryToSkin(){
		switch(selectedEntry){
			case Window:
				saveImageControlEntryToSkin(skin.getWindow());
			break;
		default:
			break;
		}
	}
	
	/**
	 * Saves the selected TextControl entry to the Skin object.
	 */
	private void saveTextControlEntryToSkin(){
		switch (selectedEntry){
		default:
			break;
		}
	}
	
	/**
	 * Saves the specified TextControl to the Skin object.
	 * @param textControl The TextControl to save to.
	 */
	private void saveTextControlEntryToSkin(TextControl textControl){
		int rowIndex = this.tblSettings.getSelectedRow();
		boolean enable = (rowIndex > -1);
		if (enable){
			if (currentTextControl != null){
				textControlValueType = TextControlValueTypes.fromInteger(rowIndex + 1);
				switch (textControlValueType){
				case Location:
					textControl.setLocation(pointPanel.getValue());
					break;
				case Size:
					textControl.setSize(dimensionPanel.getValue());
					break;
				case BackColor:
					textControl.setBackColor(colorPanel.getValue());
					break;
				case ForeColor:
					textControl.setForeColor(colorPanel.getValue());
					break;
				case Font:
					textControl.setFont(fontPanel.getValue());
					break;
				case Text:
					break;
				case Visible:
					textControl.setVisible(boolPanel.getValue());
					break;
				default:
					break;
				}
			}
		}
	}
	
	/**
	 * Saves the specified ImageControl to the Skin object.
	 * @param imageControl The ImageControl to save to.
	 */
	private void saveImageControlEntryToSkin(ImageControl imageControl){
		int rowIndex = this.tblSettings.getSelectedRow();
		boolean enable = (rowIndex > -1);
		if (enable){
			if (currentImageControl != null){
				imageControlValueType = ImageControlValueTypes.fromInteger(rowIndex + 1);
				switch (imageControlValueType){
				case Location:
					imageControl.setLocation(pointPanel.getValue());
					break;
				case Size:
					imageControl.setSize(dimensionPanel.getValue());
					break;
				case BackColor:
					imageControl.setBackColor(colorPanel.getValue());
					break;
				case ForeColor:
					imageControl.setForeColor(colorPanel.getValue());
					break;
				case Font:
					imageControl.setFont(fontPanel.getValue());
					break;
				case Text:
					break;
				case Visible:
					imageControl.setVisible(boolPanel.getValue());
					break;
				case Image:
					imageControl.setImage(imagePanel.getValue());
					break;
				default:
					break;
				}
			}
		}
	}

	/**
	 * Saves the selected Button entry to the Skin object.
	 */
	private void saveButtonEntryToSkin(){
		switch(selectedEntry){
		case Play:
			saveButtonEntryToSkin(skin.getPlay());
			break;
		case Pause:
			saveButtonEntryToSkin(skin.getPause());
			break;
		case Stop:
			saveButtonEntryToSkin(skin.getStop());
			break;
		case FastForward:
			saveButtonEntryToSkin(skin.getFastForward());
			break;
		case Rewind:
			saveButtonEntryToSkin(skin.getRewind());
			break;
		case Skip:
			saveButtonEntryToSkin(skin.getSkip());
			break;
		case Restart:
			saveButtonEntryToSkin(skin.getRestart());
			break;
		case PlayList:
			saveButtonEntryToSkin(skin.getPlayList());
			break;
		default:
			break;
		}
	}

	/**
	 * Saves the specified Button to the Skin object.
	 * @param button The Button to save to.
	 */
	private void saveButtonEntryToSkin(Button button){
		int rowIndex = this.tblSettings.getSelectedRow();
		boolean enable = (rowIndex > -1);
		if (enable){
			if (currentButton != null){
				buttonValueType = ButtonValueTypes.fromInteger(rowIndex + 1);
				switch (buttonValueType){
				case Location:
					button.setLocation(pointPanel.getValue());
					break;
				case Size:
					button.setSize(dimensionPanel.getValue());
					break;
				case BackColor:
					button.setBackColor(colorPanel.getValue());
					break;
				case ForeColor:
					button.setForeColor(colorPanel.getValue());
					break;
				case Font:
					button.setFont(fontPanel.getValue());
					break;
				case Text:
					break;
				case Visible:
					button.setVisible(boolPanel.getValue());
					break;
				case Image:
					button.setImage(imagePanel.getValue());
					break;
				case Mask:
					button.setMask(imagePanel.getValue());
					break;
				default:
					break;
				}
			} 
		}
	}
	
	/**
	 * Navigates to the specified panel in the cardPanel.
	 * @param panelType
	 */
	private void cardPanelGoToIndex(PanelTypes panelType){
		CardLayout layout = (CardLayout)cardPanel.getLayout();
		cardPanel.setVisible(false);
		layout.first(cardPanel);
		for (int i = 0; i < panelType.getIndex(); i++){
			layout.next(cardPanel);
		}
		cardPanel.setVisible(true);
	}

	/**
	 * Creates a new skin.
	 */
	private void createNewSkin(){
		skin = new configuration.Skin();
		setSkinNameToTitle();
	}

	/**
	 * Lets the user choose and load a skin.
	 */
	private void skinLoad(){
		String name;
		SkinLoadDialog skinLoadDialog = new SkinLoadDialog();
		if (skinLoadDialog.showDialog()  == JFileChooser.APPROVE_OPTION){
			name = skinLoadDialog.getSelectedFile();
			try {				
				this.skin = Skin.load(name);
				setSkinNameToTitle();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, "An error occurred when trying to load the skin '" + name +"'.", "Error loading skin", JOptionPane.WARNING_MESSAGE);
				e.printStackTrace();
			}
		}
	}	
	
	/**
	 * Saves a skin.
	 */
	private void skinSave(){
		if (skin.getName() == null || skin.getName().length() == 0){
			skinSaveAs();
		} else {
			saveSkin();
		}
	}

	/**
	 * Saves a skin to the name the user specifies.
	 */
	private void skinSaveAs(){
		SkinSaveDialog skinSaveDialog = new SkinSaveDialog();
		if (skinSaveDialog.showDialog() == JFileChooser.APPROVE_OPTION){
			skin.setName(skinSaveDialog.getSelectedFile());
			setSkinNameToTitle();
			saveSkin();
		}
	}
	
	/**
	 * Saves the skin.
	 */
	private void saveSkin() {
		try {
			skin.save();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "An error occurred when trying to save the skin '" + skin.getName() +"'.", "Error saving skin", JOptionPane.WARNING_MESSAGE);
			e.printStackTrace();
		}
	}
	
	/**
	 * Modifies the title bar with the skin name.
	 */
	private void setSkinNameToTitle(){
		String skinName;
		if (skin.getName() != null && skin.getName().length() > 0){
			skinName = skin.getName();
		} else {
			skinName = "* UNNAMED *";
		}
		this.setTitle("Skin-Editor for MP3Player - " + skinName);
	}
	
	/**
	 * Opens the Info dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmInfoActionPerformed(ActionEvent e) {
		Info info = new Info();
		info.setModal(true);
		info.setVisible(true);
	}
}