package editor;

import configuration.TextControl;

/**
 * The class TextControlTableModel inherits from EmptyTableModel and is used to show the settings of a TextControl object.
 * @author Martin Henke
 *
 */
public class TextControlTableModel extends EmptyTableModel{

	private static final long serialVersionUID = -220582455982551887L;

	/**
	 * The enum TextControlValueTypes defines the possible settings for TextControl objects.
	 */
	public static enum TextControlValueTypes {
		NONE, Location, Size, BackColor, ForeColor, Font, Text, Visible;

		public static TextControlValueTypes fromInteger(int value){
			switch (value){
			case 0:
				return NONE;
			case 1:
				return Location;
			case 2:
				return Size;
			case 3:
				return BackColor;
			case 4:
				return ForeColor;
			case 5:
				return Font;
			case 6:
				return Text;
			case 7:
				return Visible;
			default:
				return null;
			}
		}
	};
	private TextControl textControl;
	
	/**
	 * Sets the TextControl object to provide the settings.
	 * @param textControl The TextControl object to provide the settings. 
	 */
	public void setData(TextControl textControl){
		this.textControl = textControl;
	}
	
	/**
	 * Returns the number of settings (always 7).
	 */
	@Override
	public int getRowCount() {
		return 7;
	}

	/**
	 * Returns the data to show in the table.
	 * @param rowIndex The row in which the data are located.
	 * @param columnIndex The columnIndex which specifies the type of data. columnIndex=0 defines the object name, columnIndex=1 defines the setting.
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex){
		case 0:
			return getObjectName(TextControlValueTypes.fromInteger(rowIndex + 1));
		default:
			return getObjectValue(TextControlValueTypes.fromInteger(rowIndex + 1));
		}
	}
	
	/**
	 * Sets the value in the table. Editing can only take place if columnIndex=1 and is only allowed for strings.
	 */
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex){
		if (columnIndex == 1){
			TextControlValueTypes valueType = TextControlValueTypes.fromInteger(rowIndex + 1);		
			switch (valueType){
			case Location: 	// No editing in table
				break;
			case Size: 		// No editing in table
				break;
			case BackColor:	// No editing in table
				break;
			case ForeColor:	// No editing in table
				break;
			case Font:		// No editing in table
				break;
			case Text:		// No editing in table
				textControl.setText((String) aValue);
				break;
			case Visible:	// No editing in table
				break;
			case NONE:
			default:		// Illegal value
				break;
			}		
		}
	}	
	
	/**
	 * Returns if a cell is editable. Only cells with columnIndex=1 can be edited if they hold a String setting.
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex){
		if (columnIndex != 1){
			return false;
		} else {
			TextControlValueTypes valueType = TextControlValueTypes.fromInteger(rowIndex + 1);
			switch (valueType){
			case Location:
				return false;
			case Size:
				return false;
			case BackColor:
				return false;
			case ForeColor:
				return false;
			case Font:
				return false;
			case Text:
				return true;
			case Visible:
				return false;
			case NONE:
			default:
				return false;
			
			}
		}
	}
	
	/**
	 * Gets the name of a setting.
	 * @param valueType The enum value to get the setting name for.
	 * @return The name of the setting.
	 */
	private String getObjectName(TextControlValueTypes valueType){
		switch (valueType){
		case Location:
			return "Location";
		case Size:
			return "Size";
		case BackColor:
			return "BackColor";
		case ForeColor:
			return "ForeColor";
		case Font:
			return "Font";
		case Text:
			return "Text";
		case Visible:
			return "Visible";
		default:
			return null;
		}
	}
	
	/**
	 * Gets the String representation of a setting.
	 * @param valueType The enum value to get the setting value for.
	 * @return The String representation of the setting. 
	 */
	private String getObjectValue(TextControlValueTypes valueType){
		if (textControl != null){
			switch (valueType){
			case Location:
				return Formatter.format(textControl.getLocation());
			case Size:
				return Formatter.format(textControl.getSize());
			case BackColor:
				return Formatter.format(textControl.getBackColor());
			case ForeColor:
				return Formatter.format(textControl.getForeColor());
			case Font:
				return Formatter.format(textControl.getFont());
			case Text:
				return textControl.getText();
			case Visible:
				return Formatter.format(textControl.isVisible());
			default:
				return null;
			}
		} else {
			return null;
		}
	}

}
