package configuration;

import java.io.Serializable;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

/**
 * The class Control is the base class for all control settings and allows XML serialization.
 * @author Martin Henke
 *
 */
public class Control implements Serializable {

	private static final long serialVersionUID = 564060670114700714L;
	
	private Color backColor = new Color(java.awt.Color.white);
	private Font font = new Font("Thoma", Font.PLAIN, 11);
	private Color foreColor = new Color(java.awt.Color.black);
	private Point location = new Point(0, 0);
	private String name;
	private Dimension size = new Dimension(1, 1);	
	private boolean visible = true;

	/**
	 * The empty constructor is required for XML serialization and should not be used in your code.
	 */
	public Control() { }

	/**
	 * The constructor allows to specify this name as a parameter.
	 * @param name The name of this object.
	 */
	public Control(String name) {
		this.name = name;
	}

	/**
	 * Gets the background.
	 * @return The color of the background.
	 */
	public Color getBackColor() { return backColor; }
	/**
	 * Sets the background.
	 * @param backColor The new color of the background.
	 */
	@XmlElement(name = "BackColor", required = true)
	public void setBackColor(Color backColor) { this.backColor = backColor; }

	/**
	 * Gets the font.
	 * @return The font of the control.
	 */
	public Font getFont() { return font; }
	/**
	 * Sets the font.
	 * @param font The new font.
	 */
	@XmlElement(name = "Font", required = true)
	public void setFont(Font font){ this.font = font; }

	/**
	 * Gets the foreground.
	 * @return The color of the foreground.
	 */
	public Color getForeColor() { return foreColor; }
	/**
	 * Sets the foreground.
	 * @param foreColor The new color of the foreground.
	 */
	@XmlElement(name = "ForeColor", required = true)
	public void setForeColor(Color foreColor) { this.foreColor = foreColor; }
	
	/**
	 * Gets the location.
	 * @return The location of the control.
	 */
	public Point getLocation() { return location; }
	/**
	 * Sets the location.
	 * @param location The new location.
	 */
	@XmlElement(name = "Location", required = true)
	public void setLocation(Point location) { this.location = location; }
	
	/**
	 * Gets the name.
	 * @return The name of this object.
	 */
	public String getName() { return this.name; }
	
	/**
	 * Sets the name.
	 * @param name The name of this object. The name is used for identifying associated files (e.g. images in ImageControl or Button).
	 */
	@XmlAttribute(name="name", required=true)
	public void setName(String name) { this.name = name; }
	
	/**
	 * Gets the size.
	 * @return The size of the control.
	 */
	public Dimension getSize() { return size; }	
	/**
	 * Sets the size.
	 * @param size The new size.
	 */
	@XmlElement(name = "Size", required = true)
	public void setSize(Dimension size) { this.size = size; }
	
	/**
	 * Gets the visibility.
	 * @return The visibility.
	 */
	public boolean isVisible() { return visible; }
	
	/**
	 * Set the visibility.
	 * @param visible The new visibility.
	 */
	@XmlAttribute(name="visible", required=true)
	public void setVisible(boolean visible) { this.visible = visible; }
}
