package configuration;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import javax.xml.bind.annotation.XmlTransient;

/**
 * The class ImageControl extends <code>TextControl</code> and allows XML serialization of the settings for the controls with an image displayed by MP3Player.
 *  
 * @author Martin Henke
 *
 */
public class ImageControl extends TextControl {
	
	private static final long serialVersionUID = -3884044528539946469L;

	private transient Image image = null;

	/**
	 * The empty constructor is required for XML serialization and should not be used in your code.
	 */
	public ImageControl(){ 
		super();
	}

	/**
	 * The constructor allows to specify this name as a parameter.
	 * @param name The name of this object.
	 */
	public ImageControl(String name){
		super(name);
	}	

	/**
	 * Gets the image.
	 * @return The image.
	 */
	public Image getImage() { return image; }
	/**
	 * Sets the image. The image is not serialized.
	 * @param image The new image.
	 */
	@XmlTransient
	public void setImage(Image image) { this.image = image; }
	
	/**
	 * Loads the image in the specified directory.
	 * @param directory The directory which contains the image.
	 */
	public void loadImage(String directory){
		File f = getImageFile(directory, "Image");
		if (f.exists() && !f.isDirectory()){
			image = new Image(f.getAbsolutePath());
		} else {
			image = null;
		}
	}
	
	/**
	 * Copies the image to the specified directory.
	 * @param directory The directory to copy to.
	 * @throws IOException
	 */
	public void copyImage(String directory) throws IOException {
		if (image != null){
			File f = getImageFile(directory, "Image");
			copyImage(image, f);
		}
	}
	
	/**
	 * Gets the full path for the image.
	 * @param directory The directory in which the image resides.
	 * @param function The function to use within the file name. Currently this is "Image" and "Mask".
	 * @return Returns the full path to the image.
	 */
	protected File getImageFile(String directory, String function){
		return new File(directory, this.getName() + function + ".png");
	}
	
	/**
	 * Copies the image.
	 * @param img The source image.
	 * @param file The target file.
	 * @throws IOException
	 */
	protected void copyImage(Image img, File file) throws IOException{
		String sourcePath = img.getFileName();
		String targetPath = file.getAbsolutePath();
		if (!sourcePath.toLowerCase().equals(targetPath.toLowerCase())){
			Files.copy(Paths.get(sourcePath), Paths.get(targetPath), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
		}
	}
}
