package configuration;

import generic.Helper;

import java.io.*;

import javax.swing.JOptionPane;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * The Skin class is XML serializable and holds all settings for the skin. It DOES NOT hold images. Images need to be in the directory in the associated directory
 * (same name as the skin file, but without the ".skin" extension. 
 * @author Martin Henke
 *
 */
@XmlRootElement(name = "Skin")
public class Skin implements Serializable {

	private static final long serialVersionUID = 8279009055687173017L;

	/**
	 * Holds the location of the "skins" directory.
	 */
	public static final String SKIN_DIRECTORY = getCreateSkinsDirectory();
	
	/**
	 * The extension to identify skin files.
	 */
	public static final String EXTENSION = ".skin";
	
	private String name = null;
	private ImageControl window = new ImageControl("window");
	private Button play = new Button("play");
	private Button pause = new Button("pause");
	private Button stop = new Button("stop");
	private Button fastForward = new Button("fastForward");
	private Button rewind = new Button("rewind");
	private Button skip = new Button("skip");
	private Button restart = new Button("restart");
	private Button playList = new Button("playList");
	private Control menu = new Control("menu");
	private Control title = new Control("title");
	private Control elapsedTime = new Control("elapsedTime");
	
	/**
	 * The empty constructor is required for XML serialization and should not be used in your code.
	 */
	public Skin(){ }
	
	/**
	 * The constructor allows to specify the skin name as a parameter.
	 * @param name The name of the Skin.
	 */
	public Skin (String name){
		this.name = name;
	}

	/**
	 * Gets the name of the skin.
	 * @return The name of the skin.
	 */
	public String getName(){ return this.name; }
	/**
	 * Sets the name of the skin.
	 * @param name The new name of the skin.
	 */
	@XmlAttribute(name = "name", required = true)
	public void setName(String name) { this.name = name; } 

	/**
	 * Gets the Window settings.
	 * @return The window settings.
	 */
	public ImageControl getWindow() { return window;}
	/**
	 * Sets the window settings.
	 * @param window The window settings.
	 */
	@XmlElement(name = "Window", required = true)
	public void setWindow(ImageControl window) { this.window = window; }

	/**
	 * Gets the the Play button settings.
	 * @return The Play button settings.
	 */
	public Button getPlay() { return play; }
	/**
	 * Sets the Play button settings.
	 * @param play The Play button settings.
	 */
	@XmlElement(name = "Play", required = true)
	public void setPlay(Button play) { this.play = play; }

	/**
	 * Gets the Pause button settings.
	 * @return The Pause button settings.
	 */
	public Button getPause() { return pause; }
	/**
	 * Sets the Pause button settings.
	 * @param pause The Pause button settings.
	 */
	@XmlElement(name = "Pause", required = true)
	public void setPause(Button pause) { this.pause = pause; }

	/**
	 * Gets the Stop button settings.
	 * @return The Stop button settings.
	 */
	public Button getStop() { return stop; }
	/**
	 * Sets the Stop button settings.
	 * @param stop The Stop button settings.
	 */
	@XmlElement(name="Stop", required = true)
	public void setStop(Button stop) { this.stop = stop; }

	/**
	 * Gets the Fast Forward button settings.
	 * @return The Fast Forward button settings.
	 */
	public Button getFastForward() { return fastForward; }
	/**
	 * Sets the Fast Forward button settings.
	 * @param fastForward The Fast Forward button settings.
	 */
	@XmlElement(name = "FastForward", required = true)
	public void setFastForward(Button fastForward) { this.fastForward = fastForward; }

	/**
	 * Gets the Rewind button settings.
	 * @return The Rewind button settings.
	 */
	public Button getRewind() { return rewind; }
	/**
	 * Sets the Rewind button settings.
	 * @param rewind The Rewind button settings.
	 */
	@XmlElement(name = "Rewind", required = true)
	public void setRewind(Button rewind) { this.rewind = rewind; }

	/**
	 * Gets the Skip button settings.
	 * @return The Skip button settings.
	 */
	public Button getSkip() { return skip; }
	/**
	 * Sets the Skip button settings.
	 * @param skip The Skip button settings.
	 */
	@XmlElement(name = "Skip", required = true)
	public void setSkip(Button skip) { this.skip = skip; }

	/**
	 * Gets the Restart button settings.
	 * @return The Restart button settings.
	 */
	public Button getRestart() { return restart; }
	/**
	 * Sets the Restart button settings.
	 * @param restart The Restart button settings.
	 */
	@XmlElement(name = "Restart", required = true)
	public void setRestart(Button restart) { this.restart = restart; }

	/**
	 * Gets the Play button settings.
	 * @return The Play button settings.
	 */
	public Button getPlayList() { return playList; }
	/**
	 * Sets the Play button settings.
	 * @param playList The Play button settings.
	 */
	@XmlElement(name = "PlayList", required = true)
	public void setPlayList(Button playList) { this.playList = playList; }

	/**
	 * Gets the Menu settings.
	 * @return The Menu settings.
	 */
	public Control getMenu() { return menu; }
	/**
	 * Sets the Menu settings.
	 * @param menu The Menu settings.
	 */
	@XmlElement(name = "Menu", required = true)
	public void setMenu(Control menu) { this.menu = menu; }

	/**
	 * Gets the Title display settings.
	 * @return The Title display settings.
	 */
	public Control getTitle() { return title; }
	/**
	 * Sets the Title display settings.
	 * @param title The Title display settings.
	 */
	@XmlElement(name = "Title", required = true)
	public void setTitle(Control title) { this.title = title; }

	/**
	 * Gets the Elapsed Time display settings.
	 * @return The Elapsed Time display settings.
	 */
	public Control getElapsedTime() { return elapsedTime; }
	/**
	 * Sets the Elapsed Time display settings.
	 * @param elapsed The Elapsed Time display settings.
	 */
	@XmlElement(name = "ElapsedTime", required = true)
	public void setElapsedTime(Control elapsed) { this.elapsedTime = elapsed; }
	
	/**
	 * Loads the skin.
	 * @param name The name of the skin.
	 * @return Returns a Skin object.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @throws JAXBException
	 */
	public static Skin load(String name) throws IOException, ClassNotFoundException, JAXBException{
		Skin retVal;
		File inputDirectory = new File(SKIN_DIRECTORY, name);

		retVal = deserializeMe(name);
		retVal.setName(name);
		if (inputDirectory.exists() && inputDirectory.isDirectory()) {
			retVal.loadImages(inputDirectory.getAbsolutePath());
		}
		return retVal;
	}
	
	/**
	 * Deserializes the skin file for the specified skin name.
	 * @param name The name of the skin
	 * @return Returns a Skin object.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @throws JAXBException
	 */
	private static Skin deserializeMe(String name) throws IOException, ClassNotFoundException, JAXBException{
		Skin retVal;		
		JAXBContext jc = JAXBContext.newInstance(Skin.class);
	    Unmarshaller unmarshaller = jc.createUnmarshaller();
		File inputFile = new File(SKIN_DIRECTORY, name + EXTENSION);
	    
		retVal = (Skin) unmarshaller.unmarshal(inputFile);
		
		return retVal;
	}
	
	/**
	 * Loads the images into the settings of image bearing setting objects.
	 * @param directory The directory where the images are located.
	 */
	private void loadImages(String directory){
		loadImages(window, directory);
		loadImages(play, directory);
		loadImages(pause, directory);
		loadImages(stop, directory);
		loadImages(fastForward, directory);
		loadImages(rewind, directory);
		loadImages(skip, directory);
		loadImages(restart, directory);
		loadImages(playList, directory);
	}
	
	/**
	 * Loads an image into the specified ImageControl object.
	 * @param imageControl The ImageControl to hold the image.
	 * @param directory The directory in which the image is located.
	 */
	private void loadImages(ImageControl imageControl, String directory){
		imageControl.loadImage(directory);
	}
	/**
	 * Loads an image and a mask into the specified Button object.
	 * @param button The button to hold image and mask.
	 * @param directory The directory in which the image is located.
	 */
	private void loadImages(Button button, String directory){
		button.loadImage(directory);
		button.loadMask(directory);
	}

	/**
	 * Saves the skin and the associated images into a separate directory.
	 * @throws NullPointerException
	 * @throws IOException
	 * @throws JAXBException
	 */
	public void save() throws NullPointerException, IOException, JAXBException{
		// Check if a name has been specified
		if (name != null && name.length() > 0){
			// Get the file name for the skin file
			File outputFile = new File(SKIN_DIRECTORY, name + EXTENSION);
			
			// Get the directory name for the image directory 
			File outputDirectory = new File(SKIN_DIRECTORY, name);
			// Try to create the image directory
			if (outputDirectory != null && !outputDirectory.exists()){
				if (!outputDirectory.mkdir()){
					// If creation of image directory fails, throw IOException with some info.
					throw new java.io.IOException("Unable to create directory '" + name + "' in directory '" + SKIN_DIRECTORY + "'");
				}
			}
			// Save the images into the image directory.
			saveImages(outputDirectory.getAbsolutePath());
			
			// Serialize the Skin object
			serializeMe(outputFile);
		} else {
			// If no name has been specified, throw a NullPointerException.
			throw new NullPointerException("Name must not be null or empty.");
		}
	}
	
	/**
	 * Serializes the Skin object into a XML file.
	 * @param outputFile The File object pointing to the location of the (to be created) XML file.
	 * @throws IOException
	 * @throws JAXBException
	 */
	private void serializeMe(File outputFile) throws IOException, JAXBException{
		JAXBContext context = JAXBContext.newInstance(Skin.class);		
	    Marshaller marshaller = context.createMarshaller();
	    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
	    marshaller.marshal(this, outputFile);
	}
	
	/**
	 * Saves the images into the specified directory.
	 * @param directory The directory to save the images to.
	 * @throws IOException
	 */
	private void saveImages(String directory) throws IOException{
		saveImages(window, directory);
		saveImages(play, directory);
		saveImages(pause, directory);
		saveImages(stop, directory);
		saveImages(fastForward, directory);
		saveImages(rewind, directory);
		saveImages(skip, directory);
		saveImages(restart, directory);
		saveImages(playList, directory);
	}
	
	/**
	 * Saves the image of the specified ImageControl object into the specified directory.
	 * @param imageControl The ImageControl object holding the image.
	 * @param directory The directory to save the image to.
	 * @throws IOException
	 */
	private void saveImages(ImageControl imageControl, String directory) throws IOException{
		imageControl.copyImage(directory);
	}
	/**
	 * Saves image and mask of the specified Button object into the specified directory.
	 * @param button The Button object holding image and mask.
	 * @param directory The directory to save the image to.
	 * @throws IOException
	 */
	private void saveImages(Button button, String directory) throws IOException{
		button.copyImage(directory);
		button.saveMask(directory);
	}
	
	/**
	 * Shows if the skin with the specified file name exists.
	 * @param fileName The file name of the skin.
	 * @return Returns if the skin file exists.
	 */
	public static boolean skinExists(String fileName){
		File skin = new File(SKIN_DIRECTORY, fileName);
		return (skin.exists() && !skin.isDirectory());		
	}
	
	/**
	 * Creates the "skins" directory if it does not exist and returns the full path.
	 * @return
	 */
	private static String getCreateSkinsDirectory(){
		String retVal = null;
		File skinDir;
		
		try {
			// Get the path to the "skins" directory.
			skinDir = new File(Helper.getJarDirectory(), "skins");			
			retVal = skinDir.getAbsolutePath();
			// Check if the directory already exists.
			if (!skinDir.exists()){
				if (!skinDir.mkdirs()){
					// Creation of directory failed --> throw an IOException.
					throw new java.io.IOException("Unable to create directory '" + retVal + "'");
				}
			}
		} catch (IOException e) {
			// Show an error message to the user and exit.
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error creating Skins directory", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		// Return the full path to the "skins" directory
		return retVal;
	}
}
