package generic;
import java.awt.*;
import javax.swing.*;

/**
 * @author Rob Cormick
 * @since 2008-10-12
 * {@link http://tips4java.wordpress.com/2008/10/12/background-panel/}
 * 
 * The author allows to use/modify the code freely. The permission was given on http://tips4java.wordpress.com/about/ and was online on 2013-04-10.
 *  
 *  Support custom painting on a panel in the form of
 *
 *  a) images - that can be scaled, tiled or painted at original size
 *  b) non solid painting - that can be done by using a Paint object
 *
 *  Also, any component added directly to this panel will be made non-opaque so that the custom painting can show through.
 *  
 *  Modifications:
 *  ==============
 *  1. Constants ACTUAL, TILED and SCALED have been replaced by an enum.
 *  2. Added an empty constructor
 *  3. Created getters for the member variables which have setters
 *  4. Changed comments to Javadoc style
 *  5. Minor code formatting
 */
public class BackgroundPanel extends JPanel
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 968244390436072157L;

	public static enum Style {
		Actual, Tiled, Scaled
	};
	
	private Paint painter = null;
	private Image image = null;
	private Style style = Style.Actual;
	private float alignmentX = 0.0f;
	private float alignmentY = 0.0f;
	private boolean isTransparentAdd = true;

	/**
	 * The empty constructor is required for the WindowBuilder.
	 */
	public BackgroundPanel(){
		this(null, Style.Actual);
	}
	
	/**
	 *  Set image as the background with the SCALED style
	 * @param image The image to use for painting the background
	 */
	public BackgroundPanel(Image image)
	{
		this(image, Style.Actual);
	}

	/**
	 *  Sets image as the background with the specified style
	 * @param image The image to use for painting the background
	 * @param style The style to use for painting the background
	 */
	public BackgroundPanel(Image image, Style style)
	{
		setImage( image );
		setStyle( style );
		setLayout( new BorderLayout() );
	}

	/**
	 *  Sets image as the background with the specified style and alignments
	 * @param image The image to use for painting the background
	 * @param style The style to use for painting the background
	 * @param alignmentX The horizontal alignment for "Actual" style
	 * @param alignmentY The vertical alignment for "Actual" style
	 */
	public BackgroundPanel(Image image, Style style, float alignmentX, float alignmentY)
	{
		setImage(image);
		setStyle(style);
		setImageAlignmentX( alignmentX );
		setImageAlignmentY( alignmentY );
		setLayout( new BorderLayout() );
	}

	/**
	 *  Sets the Paint interface to paint a background
	 *  @param painter The Paint interface to paint a background
	 */
	public BackgroundPanel(Paint painter)
	{
		setPaint(painter);
		setLayout(new BorderLayout());
	}

	/**
	 * Gets the image used as the background
	 * @return The image used as the background
	 */
	public Image getImage(){ return this.image; }
	

	/**
	 * Set the image used as the background
	 * @param image The image used as the background
	 */
	public void setImage(Image image)
	{
		this.image = image;
		repaint();
	}

	/**
	 * Gets the style used to paint the background
	 * @return The style used to paint the background
	 */
	public Style getStyle() { return this.style; }
	
	/**
	 *	Sets the style used to paint the background image
	 * @param style The style used to paint the background image 
	 */
	public void setStyle(Style style)
	{
		this.style = style;
		repaint();
	}

	/**
	 * Gets the Paint object used to paint the background
	 * @return The Paint object used to paint the background
	 */
	public Paint getPaint(){ return this.painter; }
	
	/**
	 *	Sets the Paint object used to paint the background
	 * @param painter The Paint object used to paint the background 
	 */
	public void setPaint(Paint painter)
	{
		this.painter = painter;
		repaint();
	}

	/**
	 * Gets the horizontal alignment of the image when using "Actual" style
	 * @return The horizontal alignment of the image when using "Actual" style
	 */
	public float getImageAlignmentX() { return this.alignmentX; }
	
	/**
	 * Sets the horizontal alignment of the image when using "Actual" style
	 * @param alignmentX The horizontal alignment of the image when using "Actual" style
	 */
	public void setImageAlignmentX(float alignmentX)
	{
		this.alignmentX = alignmentX > 1.0f ? 1.0f : alignmentX < 0.0f ? 0.0f : alignmentX;
		repaint();
	}

	/**
	 * Gets the vertical alignment of the image when using "Actual" style
	 * @return The vertical alignment of the image when using "Actual" style
	 */
	public float getImageAlignmentY() { return this.alignmentY; }
	
	/**
	 * Sets the vertical alignment of the image when using "Actual" style
	 * @param alignmentY The vertical alignment of the image when using "Actual" style
	 */
	public void setImageAlignmentY(float alignmentY)
	{
		this.alignmentY = alignmentY > 1.0f ? 1.0f : alignmentY < 0.0f ? 0.0f : alignmentY;
		repaint();
	}

	/**
	 * Adds a component to the panel. Overriden as it is required to make the component transparent.
	 * @param component The component to be added to the panel.
	 */
	public void add(JComponent component)
	{
		add(component, null);
	}

	/**
	 *  Gets the preferred size. Overriden to provide a preferred size equal to the image size.
	 *  @return The preferred size
	 */
	@Override
	public Dimension getPreferredSize()
	{
		if (image == null)
			return super.getPreferredSize();
		else
			return new Dimension(image.getWidth(null), image.getHeight(null));
	}

	/**
	 * Adds a component with the specified constraints to the panel. Overriden as it is required to make the component transparent.
	 * @param component The component to be added to this panel.
	 * @param constraints The constraints of the component.
	 */
	public void add(JComponent component, Object constraints)
	{
		if (isTransparentAdd)
		{
			makeComponentTransparent(component);
		}

		super.add(component, constraints);
	}

	/**
	 *  Controls whether components added to this panel should automatically be made transparent. If this is the case, setOpaque(false) will be invoked.
	 *  The default is set to true.
	 * @param isTransparentAdd Specifies if the component should be made transparent.
	 */
	public void setTransparentAdd(boolean isTransparentAdd)
	{
		this.isTransparentAdd = isTransparentAdd;
	}

	/**
	 *  Try to make the component transparent. For components that use renderers, like JTable, you will also need to change the renderer to be transparent.
	 *  An easy way to do this it to set the background of the table to a Color using an alpha value of 0.
	 * @param component The component to make transparent.
	 */
	private void makeComponentTransparent(JComponent component)
	{
		component.setOpaque(false);

		if (component instanceof JScrollPane)
		{
			JScrollPane scrollPane = (JScrollPane)component;
			JViewport viewport = scrollPane.getViewport();
			viewport.setOpaque(false);
			Component c = viewport.getView();

			if (c instanceof JComponent)
			{
				((JComponent)c).setOpaque(false);
			}
		}
	}

	/**
	 *  Adds custom painting
	 *  @param g The graphics interface to use for painting 
	 */
	@Override
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		//  Invoke the painter for the background

		if (painter != null)
		{
			Dimension d = getSize();
			Graphics2D g2 = (Graphics2D) g;
			g2.setPaint(painter);
			g2.fill( new Rectangle(0, 0, d.width, d.height) );
		}

		//  Draw the image

		if (image == null) return;

		switch (style)
		{
			case Scaled:
				drawScaled(g);
				break;

			case Tiled:
				drawTiled(g);
				break;

			case Actual:
				drawActual(g);
				break;

			default:
            	drawScaled(g);
		}
	}

	/**
	 *  Custom painting code for drawing a "Scaled" image as the background
	 *  @param g The graphics interface to use for painting 
	 */
	private void drawScaled(Graphics g)
	{
		Dimension d = getSize();
		g.drawImage(image, 0, 0, d.width, d.height, null);
	}

	/**
	 *  Custom painting code for drawing "Tiled" images as the background
	 *  @param g The graphics interface to use for painting 
	 */
	private void drawTiled(Graphics g)
	{
		   Dimension d = getSize();
		   int width = image.getWidth(null);
		   int height = image.getHeight(null);

		   for (int x = 0; x < d.width; x += width)
		   {
			   for (int y = 0; y < d.height; y += height)
			   {
				   g.drawImage( image, x, y, null, null );
			   }
		   }
	}

	/**
	 *  Custom painting code for drawing the "Actual" image as the background. The image is positioned in the panel based on the horizontal and vertical alignments
	 *  specified.
	 *  @param g The graphics interface to use for painting 
	 */
	private void drawActual(Graphics g)
	{
		Dimension d = getSize();
		Insets insets = getInsets();
		int width = d.width - insets.left - insets.right;
		int height = d.height - insets.top - insets.left;
		float x = (width - image.getWidth(null)) * alignmentX;
		float y = (height - image.getHeight(null)) * alignmentY;
		g.drawImage(image, (int)x + insets.left, (int)y + insets.top, this);
	}
}
